﻿using Amazon.Runtime;
using Amazon.S3;
using Amazon.S3.Model;
using System;
using System.Threading.Tasks;

namespace WasabiApi
{
    public class Wasabi
    {
        private static IAmazonS3 _s3Client;

        public Wasabi(string accessKey, string secretKey)
        {
            // 1. this is necessary for the endpoint
            var config = new AmazonS3Config { ServiceURL = "https://s3.wasabisys.com" };
            var credentials = new BasicAWSCredentials(accessKey, secretKey);
            _s3Client = new AmazonS3Client(credentials, config);
        }


        public async Task<string> UploadFile(string bucketName, string objName, string pathAndFileName)
        {
            try
            {
                var obj = $"{objName}";
                var path = $"{pathAndFileName}";
                var putRequest = new PutObjectRequest();
                putRequest.BucketName = bucketName;
                putRequest.Key = objName;
                putRequest.FilePath = pathAndFileName;

                //putRequest.Metadata.Add("x-amz-meta-title", "someTitle");
                await _s3Client.PutObjectAsync(putRequest);
                return "OK";
            }
            catch (AmazonS3Exception ex)
            {
                return ex.Message ;
            }
        }


        public async Task<string> DownloadFile(string bucketName, string objName)
        {
            try
            {
                var obj = $"{objName}";
                var getRequest = new GetObjectRequest();
                getRequest.BucketName = bucketName;
                getRequest.Key = objName;

                //putRequest.Metadata.Add("x-amz-meta-title", "someTitle");
                var response = await _s3Client.GetObjectAsync(getRequest);

                if (response.HttpStatusCode == System.Net.HttpStatusCode.OK)
                {
                    return "OK";
                }
                else
                {
                    return "Something went wrong";
                }
            }
            catch (AmazonS3Exception ex)
            {
                return ex.Message;
            }
        }

        public async Task<string> CreateBucketAsync(string bucketName, string objName)
        {
            try
            {
                PutObjectRequest request = new PutObjectRequest();
                request.BucketName = bucketName;
                request.Key = objName;

                PutObjectResponse response = await _s3Client.PutObjectAsync(request);
                return "OK";
            }

            catch (Amazon.S3.AmazonS3Exception ex)
            {
                return ex.Message;
            }
        }

        public async Task<string> CreateFolderAsync(string bucketName, string folderName)
        {
            if (await DoesBucketExistAsync(bucketName) == false)
            {
                return "Bucket does not exists";
            }
            try
            {
                PutObjectRequest request = new PutObjectRequest();
                request.BucketName = bucketName;
                request.Key = folderName + "/";

                PutObjectResponse response = await _s3Client.PutObjectAsync(request);
                return "OK";
            }

            catch (Amazon.S3.AmazonS3Exception ex)
            {
                return ex.Message;
            }
        }


        public async Task<bool> DoesBucketExistAsync(string bucketName)
        {
            try
            {
                var response = await _s3Client.DoesS3BucketExistAsync(bucketName);
                return true;
            }
            catch (Amazon.S3.AmazonS3Exception ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                //status wasn't not found, so throw the exception
                throw;
            }
        }


        /// <summary>
        /// Note:  Use the backslash "/" to denote folders...
        ///        example: "MyFolder/St. Paul Agreement - Mark Theobald.pdf"
        /// </summary>
        /// <param name="bucketName"></param>
        /// <param name="prefix"></param>
        /// <returns></returns>
        public async Task<bool> DoesObjectExistAsync(string bucketName, string prefix)
        {
            try
            {
                var obj = $"{prefix}";
                var response = await _s3Client.GetAllObjectKeysAsync(bucketName, obj, null);
                return response.Count > 0;
            }
            catch (Amazon.S3.AmazonS3Exception ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                //status wasn't not found, so throw the exception
                throw;
            }
        }





        #region Private Methods
        #endregion
    }
}
